diff -rPu --new-file ../jemplode-src-ftp/jempeg-empeg/src/org/jempeg/empeg/util/HijackUtils.java jempeg-empeg/src/org/jempeg/empeg/util/HijackUtils.java
--- ../jemplode-src-ftp/jempeg-empeg/src/org/jempeg/empeg/util/HijackUtils.java	Wed Nov 19 22:37:02 2003
+++ jempeg-empeg/src/org/jempeg/empeg/util/HijackUtils.java	Sat Nov 22 16:14:32 2003
@@ -64,6 +64,9 @@
 * @version $Revision: 1.6 $
 */
 public class HijackUtils {
+  private static final String FID_PATH0 = "/empeg/fids0/";
+  private static final String FID_PATH1 = "/empeg/fids1/";
+
   /**
   * Returns whether or not Hijack should be used to communicate
   * with the given connection.  This will check the user's
@@ -187,16 +190,11 @@
   * @throws IOException on ftp errors
   */
   public static void deleteFid(InetAddress _address, long _fid) throws IOException {
-    String _remoteFidName = StringUtils.pad(_fid, 8, 16).toLowerCase();
-    _remoteFidName = _remoteFidName.substring(0, 5) + "/" + _remoteFidName.substring(5);
-        
     FTPClient ftp = new FTPClient(_address);
     try {
 	login(ftp);
-	ftp.quote("DELE /empeg/fids0/_" + _remoteFidName, null);
-	ftp.quote("DELE /empeg/fids1/_" + _remoteFidName, null);
-	ftp.quote("DELE /empeg/fids0/" + Long.toHexString(_fid), null);
-	ftp.quote("DELE /empeg/fids1/" + Long.toHexString(_fid), null);
+	String fidPath = findFID(ftp,_fid);
+	ftp.delete(fidPath);
     } finally {
 	ftp.quit();
     }
@@ -218,25 +216,35 @@
   * @throws IOException if the upload fails
   */
   public static void uploadFid(InetAddress _address, long _fid, InputStream _is, long _initialOffset, long _size, ISimpleProgressListener _listener, boolean _newFidLayout, int _drive, Method _pingMethod, Object _pingClass) throws IOException {
-    System.out.println("Uploading fid:" + _fid + " to drive" + _drive);
-    String _remoteFidBase = _remoteFidBase = "/empeg/fids"+_drive+"/";
-    String _remoteFidName = Long.toHexString(_fid);
-    if(_newFidLayout) {
-	_remoteFidName = StringUtils.pad(_fid, 8, 16).toLowerCase();
-	_remoteFidName = "_" + _remoteFidName.substring(0, 5) + "/" + _remoteFidName.substring(5);
-    }
-    
     FTPClient ftp = new FTPClient(_address);
     try {
       login(ftp);
+      String foundFid = findFID(ftp, _fid);
+    
       OutputStream os = null;
-      try {
-	os = ftp.putBinary(_remoteFidBase + _remoteFidName, (int)_initialOffset);
-      } catch (com.enterprisedt.net.ftp.FTPException e) {
-	// If it failed we probably need to create the directory first
-	ftp.mkdir(_remoteFidBase + _remoteFidName.substring(0, 6));
-	os = ftp.putBinary(_remoteFidBase + _remoteFidName, (int)_initialOffset);
+      if(foundFid!=null) {
+		os = ftp.putBinary(foundFid, (int)_initialOffset);
+      } else {
+	String _remoteFidBase = _remoteFidBase = "/empeg/fids"+_drive+"/";
+	String _remoteFidName = Long.toHexString(_fid);
+	if(_newFidLayout) {
+	    _remoteFidName = StringUtils.pad(_fid, 8, 16).toLowerCase();
+	    _remoteFidName = "_" + _remoteFidName.substring(0, 5) + "/" + _remoteFidName.substring(5);
+	}
+	
+	try {
+	    os = ftp.putBinary(_remoteFidBase + _remoteFidName, (int)_initialOffset);
+	} catch (com.enterprisedt.net.ftp.FTPException e) {
+	    if(_newFidLayout) {
+		// If it failed we probably need to create the directory first
+		ftp.mkdir(_remoteFidBase + _remoteFidName.substring(0, 6));
+		os = ftp.putBinary(_remoteFidBase + _remoteFidName, (int)_initialOffset);
+	    } else {
+		throw e;
+	    }
+	}
       }
+      
       FileUtils.copy(_is, os, 16384, _size, 0, _size, _listener, _pingMethod, _pingClass);
       os.flush();
       os.close();
@@ -270,13 +278,55 @@
 	
     try {
 	login(ftp);
-	ftp.chdir("/empeg/fids0/_00000");
+	ftp.chdir(FID_PATH0 + "_00000");
 	return true;
     } catch (Exception e) {
 	return false;
     } finally {
 	ftp.quit();
     }
+  }
+  
+ /**
+  * Find out if a file exists.  This is kind of a hack as 553 is technically an
+  * error response.  Luckily it is a different error response than file not found.
+  * The right way to do this is apparently to parse the output of a LIST of the
+  * parent directory, but I don't want the overhead of doing that.
+  *
+  * @param _ftp the ftp connection to use
+  * @param _file the file to look for
+  * @throws IOException if ftp fails
+  */ 
+  private static boolean isExistingFile(FTPClient _ftp, String _file) {
+    try {
+	String[] validCode = { "553" };
+	_ftp.quote("NLST " + _file,validCode);
+	return true;
+    } catch (Exception e) {
+	return false;
+    }
+  }
+  
+ /**
+  * Try to find the path of an existing FID on the empeg
+  *
+  * @param _address the address to connect to
+  * @throws IOException if ftp fails
+  */ 
+  private static String findFID(FTPClient _ftp, long _fid) {
+    String _remoteFidName = StringUtils.pad(_fid, 8, 16).toLowerCase();
+    _remoteFidName = _remoteFidName.substring(0, 5) + "/" + _remoteFidName.substring(5);
+    
+    if(isExistingFile(_ftp,FID_PATH0 + "_" + _remoteFidName))
+	return FID_PATH0 + "_" + _remoteFidName;
+    else if(isExistingFile(_ftp,FID_PATH1 + "_" + _remoteFidName))
+	return FID_PATH1 + "_" + _remoteFidName;
+    else if(isExistingFile(_ftp,FID_PATH0 + Long.toHexString(_fid)))
+	return FID_PATH0 + Long.toHexString(_fid);
+    else if(isExistingFile(_ftp,FID_PATH1 + Long.toHexString(_fid)))
+	return FID_PATH1 + Long.toHexString(_fid);
+    else
+	return null;
   }
 
   /**
